import React, { useState, useEffect, useRef, useContext } from 'react'
import { ConnectionService } from 'services/connectionService'
import { TriggerService } from 'services/triggersService'
import { AccountService } from 'services/accountService'

import { AppContext } from 'components/Contexts/AppContextProvider'

import { JobService, JobState } from 'services/jobsService'
import { FluidType } from 'enum/fluid.enum'

import { withClient, Client } from 'cozy-client'
import { translate } from 'cozy-ui/react/I18n'

import { getPicto, getAddPicto, getFuildType, getParamPicto } from 'utils/utils'
import chevronDown from 'assets/icons/ico/chevron-down.svg'
import chevronUp from 'assets/icons/ico/chevron-up.svg'
import TrailingIcon from 'assets/icons/ico/trailing-icon.svg'

import StyledIcon from 'components/CommonKit/Icon/StyledIcon'
import StyledIconButton from 'components/CommonKit/IconButton/StyledIconButton'
import StyledButton from 'components/CommonKit/Button/StyledButton'

import failurePicto from '../KonnectorViewer/picto-failure.png'
import successPicto from '../KonnectorViewer/picto-success.png'
import pendingPicto from '../KonnectorViewer/picto-pending.png'
import KonnectorService from 'services/konnectorService'
import KonnectorStatusService from 'services/konnectorStatusService'
import Spinner from 'components/CommonKit/Spinner/Spinner'
import IFluidConfig from 'services/IFluidConfig'

interface KonnectorViewerCardProps {
  fluidConfig: IFluidConfig
  trigger: any | null
  client: Client
  isParam: boolean
  t: Function
}

function revealPassword(idInput: string) {
  const input = document.getElementById(idInput)
  if (input) {
    if (input.getAttribute('type') === 'password') {
      input.setAttribute('type', 'text')
    } else {
      input.setAttribute('type', 'password')
    }
  }
}

const KonnectorViewerCard: React.FC<KonnectorViewerCardProps> = ({
  fluidConfig,
  trigger,
  client,
  isParam,
  t,
}: KonnectorViewerCardProps) => {
  const [login, setLogin] = useState<string>('')
  const [password, setPassword] = useState<string>('')
  const [loading, setLoading] = useState<boolean>(false)
  const [konnectorAccountId, setAccountId] = useState<string>('')
  const [isKonnectorAcc, setAccount] = useState<boolean>(false)
  const [setActive, setActiveState] = useState('')
  const [setHeight, setHeightState] = useState('0px')
  const [updateTrigger, setUpdateTrigger] = useState<any>(null)
  const [updateDate, setUpdateDate] = useState<string>(
    new Date('0001-01-01T00:00:00Z').toDateString()
  )
  // const [frequency, setFrequency] = useState<string>('daily')
  const [error, setError] = useState<string>('')
  const [jobState, setJobState] = useState<string>('')
  const [launchedJob, setlaunchedJob] = useState<any>(null)
  const content: React.MutableRefObject<null> = useRef<null>(null)
  const type: string = fluidConfig.konnectorConfig.type
  const fluid: FluidType = getFuildType(type)
  const iconType = getPicto(fluid)
  const context = useContext(AppContext)

  const iconAddType = isParam ? getParamPicto(fluid) : getAddPicto(fluid)

  const toggleAccordion = () => {
    setActiveState(setActive === '' ? 'active' : '')
    setHeightState(
      setActive === 'active' &&
        content &&
        content.current &&
        content.current.scrollHeight
        ? '0px'
        : `${content.current.scrollHeight}px`
    )
  }

  const handleSubmit = async (e: React.FormEvent<HTMLFormElement>) => {
    try {
      e.preventDefault()
      setError('')
      setLoading(true)
      if (!login || !password) {
        setError('Please enter a login and password')
        setLoading(false)
        return null
      }
      setJobState(JobState.Running)
      const connectionService = new ConnectionService(
        client,
        fluidConfig.konnectorConfig.id,
        login,
        password
      )
      const data = await connectionService.connectNewUser()
      setLoading(false)
      if (!data) {
        setError('Error during the user connection')
        setLoading(false)
        setJobState(JobState.Errored)
        return null
      }
      trigger = await KonnectorStatusService.getSingleTrigger(
        client,
        data.trigger_id
      )
      setUpdateTrigger(trigger)
      setlaunchedJob(data)
      setUpdateDate(new Date(data.queued_at).toDateString())
      setAccountId(data.message.account)
      setError('Connected')
      setAccount(true)
      await context.refreshFluidTypes()
      return data
    } catch (error) {
      setLoading(false)
      setError(error.message)
    }
  }

  const deleteAccount = async () => {
    try {
      const account = await AccountService.getAccount(
        client,
        konnectorAccountId
      )
      if (!account) {
        setError('Error fetching account')
        setLoading(false)
      }
      const delAccount = await AccountService.deleteAccount(client, account)
      if (!delAccount) {
        setError('Error during deletion account')
        setLoading(false)
      }
      setError('')
      setJobState('')
      toggleAccordion()
      trigger = null
      setAccount(false)
      context.refreshFluidTypes()
    } catch (error) {
      setLoading(false)
      setError(error.message)
    }
  }

  const updateKonnector = async () => {
    setError('')
    setJobState('')
    setLoading(true)
    try {
      const account = await AccountService.getAccount(
        client,
        konnectorAccountId
      )
      const konnectorService = new KonnectorService(
        client,
        fluidConfig.konnectorConfig.id
      )
      const konnector = await konnectorService.fetchKonnector()
      const triggersServices = new TriggerService(client, account, konnector)
      if (updateTrigger !== null) {
        triggersServices.setTrigger(updateTrigger)
      } else {
        triggersServices.setTrigger(trigger)
      }
      const job = await triggersServices.launchTrigger()
      if (!job) {
        throw new Error(`Error during trigger launching`)
      }
      setlaunchedJob(job)
      setLoading(false)
      setUpdateDate(new Date(job.queued_at).toDateString())
    } catch (error) {
      setLoading(false)
      setError(error.message)
    }
  }

  const getIcon = (jobState: string) => {
    switch (jobState) {
      case JobState.Errored:
        return failurePicto
      // case JobState.Running:
      //   return pendingPicto
      // case JobState.Done:
      //   return successPicto
      default:
        return ''
    }
  }

  const getKonnectorStateMarkup = (jobState: string) => {
    const iconSrc = getIcon(jobState)
    if (iconSrc === '') return ''
    return <img className="state-icon" src={iconSrc}></img>
  }

  // TODO: To be removed. This code slows the job state rendering so It can be visible
  const setJobStateAfterTimer = (state: string) => {
    setTimeout(() => {
      setJobState(state)
    }, 1000)
  }

  const jobStateCallBack = async (state: string) => {
    setJobState(state)
  }

  useEffect(() => {
    let subscribed = true
    async function getTriggerData() {
      if (trigger || launchedJob) {
        const jobService = new JobService(client)
        let runningJob = null

        if (trigger) {
          let triggerState = ''
          triggerState = TriggerService.fetchStateFromTrigger(trigger)
          const konnectorAcc = TriggerService.fetchKonnectorAccountFromTrigger(
            trigger
          )
          const konnectorId = fluidConfig.konnectorConfig.id.split('/')
          if (subscribed && konnectorAcc.konnector === konnectorId[1]) {
            setAccountId(konnectorAcc.account)
            setAccount(true)
          }
          if (subscribed && !launchedJob) setJobState(triggerState.status)
          if (triggerState.status === JobState.Running)
            runningJob = jobService.fetchJob(triggerState.last_executed_job_id)
        }
        if (launchedJob) runningJob = launchedJob
        if (runningJob) jobService.watch(runningJob, setJobStateAfterTimer)
      }
    }
    getTriggerData()
    return () => {
      subscribed = false
    }
  }, [trigger, launchedJob])

  return (
    <>
      <div className={`accordion ${setActive}`}>
        <div>
          <div
            className={`accordion-header ${setActive}`}
            onClick={toggleAccordion}
          >
            <div className="accordion-icon">
              {!isKonnectorAcc ? (
                <StyledIcon className="icon" icon={iconAddType} size={49} />
              ) : (
                <StyledIcon className="icon" icon={iconType} size={49} />
              )}
            </div>
            <div className="state-picto">
              {getKonnectorStateMarkup(jobState)}
            </div>
            <div className="accordion-info">
              <div className="accordion-title text-16-normal">
                {!isKonnectorAcc
                  ? t('KONNECTORCONFIG.LABEL_CONNECTTO_' + FluidType[fluid])
                  : t('FLUID.' + FluidType[fluid] + '.LABEL')}
              </div>
            </div>
            <StyledIconButton icon={setActive ? chevronUp : chevronDown} />
          </div>
          <div
            ref={content}
            style={{ maxHeight: `${setHeight}` }}
            className={`accordion-content ${setActive}`}
          >
            {!isKonnectorAcc ? (
              <form
                className="form"
                onSubmit={(e: React.FormEvent<HTMLFormElement>) =>
                  handleSubmit(e)
                }
              >
                {t('KONNECTORCONFIG.LABEL_FILLIN')}{' '}
                {fluidConfig.konnectorConfig.name}
                <div className="form-group">
                  <div className="form-message">{error}</div>
                  <input
                    id={'idFieldLogin' + type}
                    type="text"
                    className="form-control form-input"
                    aria-describedby="emailHelp"
                    placeholder="Adresse e-mail"
                    name="login"
                    onChange={(e: React.ChangeEvent<HTMLInputElement>) =>
                      setLogin(e.target.value)
                    }
                    value={login}
                  ></input>
                </div>
                <div className="form-group">
                  <div className="form-message">{error}</div>
                  <input
                    id={'idFieldPassword' + type}
                    type="password"
                    className="form-control form-input"
                    aria-describedby="PasswordHelp"
                    placeholder="Mot de passe"
                    name="password"
                    onChange={(e: React.ChangeEvent<HTMLInputElement>) =>
                      setPassword(e.target.value)
                    }
                    value={password}
                  />
                  <span>
                    <StyledIconButton
                      icon={TrailingIcon}
                      className="form-trailing-icon"
                      size={22}
                      onClick={() => revealPassword('idFieldPassword' + type)}
                    />
                  </span>
                </div>
                <StyledButton type="submit" color="primary" disabled={loading}>
                  {t('KONNECTORCONFIG.BTN_CONNECTION')}
                </StyledButton>
                <StyledButton
                  type="button"
                  color="secondary"
                  disabled={loading}
                >
                  {t('KONNECTORCONFIG.BTN_NOACCOUNT')}
                </StyledButton>
              </form>
            ) : (
              <div>
                <div>
                  <div className="accordion-update">
                    <div
                      className={
                        JobState.Errored
                          ? 'accordion-caption-red'
                          : 'accordion-caption'
                      }
                    >
                      {t('KONNECTORCONFIG.LABEL_UPDATEDAT')}
                    </div>
                    {trigger ? (
                      <div>{TriggerService.fetchDateFromTrigger(trigger)}</div>
                    ) : (
                      <div>{updateDate}</div>
                    )}
                  </div>
                  <div className="inline-buttons">
                    <StyledButton
                      type="button"
                      color="primary"
                      onClick={updateKonnector}
                    >
                      {loading ? (
                        <Spinner size="2em" />
                      ) : (
                        <div>{t('KONNECTORCONFIG.BTN_UPDATE')}</div>
                      )}
                    </StyledButton>
                    <StyledButton
                      type="button"
                      color="secondary"
                      onClick={deleteAccount}
                      disabled={loading}
                    >
                      {t('KONNECTORCONFIG.BTN_DELETE')}
                    </StyledButton>
                  </div>
                </div>
              </div>
            )}
          </div>
        </div>
      </div>
    </>
  )
}
export default translate()(withClient(KonnectorViewerCard))
